/*
 *  Copyright Droids Corporation (2011)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Revision : $Id: commands_gen.c,v 1.8 2009-11-08 17:24:33 zer0 Exp $
 *
 *  Olivier MATZ <zer0@droids-corp.org>
 */

#include <stdio.h>
#include <string.h>

#include <aversive/pgmspace.h>
#include <aversive/wait.h>
#include <aversive/error.h>
#include <aversive/queue.h>

#include <uart.h>
#include <clock_time.h>

#include <scheduler.h>
#include <scheduler_stats.h>

#include <rdline.h>
#include <parse.h>
#include <parse_string.h>
#include <parse_num.h>

#include <diagnostic.h>

#include "callout.h"
#include "main.h"
#include "cmdline.h"

/**********************************************************/
/* Reset */

/* this structure is filled when cmd_reset is parsed successfully */
struct cmd_reset_result {
	fixed_string_t arg0;
};

/* function called when cmd_reset is parsed successfully */
static void cmd_reset_parsed(void * parsed_result, void * data)
{
	(void)parsed_result;
	(void)data;
#ifdef HOST_VERSION
	hostsim_exit();
#endif
	reset();
}

const char PROGMEM str_reset_arg0[] = "reset";
const parse_token_string_t PROGMEM cmd_reset_arg0 = TOKEN_STRING_INITIALIZER(struct cmd_reset_result, arg0, str_reset_arg0);

const char PROGMEM help_reset[] = "Reset the board";
const parse_inst_t PROGMEM cmd_reset = {
	.f = cmd_reset_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_reset,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_reset_arg0,
		NULL,
	},
};

/**********************************************************/
/* Bootloader */

/* this structure is filled when cmd_bootloader is parsed successfully */
struct cmd_bootloader_result {
	fixed_string_t arg0;
};

/* function called when cmd_bootloader is parsed successfully */
static void cmd_bootloader_parsed(void *parsed_result, void *data)
{
	(void)parsed_result;
	(void)data;
#ifndef HOST_VERSION
	bootloader();
#else
	printf("not implemented\n");
#endif
}

const char PROGMEM str_bootloader_arg0[] = "bootloader";
const parse_token_string_t PROGMEM cmd_bootloader_arg0 = TOKEN_STRING_INITIALIZER(struct cmd_bootloader_result, arg0, str_bootloader_arg0);

const char PROGMEM help_bootloader[] = "Launch the bootloader";
const parse_inst_t PROGMEM cmd_bootloader = {
	.f = cmd_bootloader_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_bootloader,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_bootloader_arg0,
		NULL,
	},
};

/**********************************************************/
/* Scheduler show */

/* this structure is filled when cmd_scheduler is parsed successfully */
struct cmd_scheduler_result {
	fixed_string_t arg0;
	fixed_string_t arg1;
};

/* function called when cmd_scheduler is parsed successfully */
static void cmd_scheduler_parsed(void *parsed_result, void *data)
{
	(void)parsed_result;
	(void)data;
	scheduler_dump_events();
	scheduler_stats_dump();
}

const char PROGMEM str_scheduler_arg0[] = "scheduler";
const parse_token_string_t PROGMEM cmd_scheduler_arg0 = TOKEN_STRING_INITIALIZER(struct cmd_scheduler_result, arg0, str_scheduler_arg0);
const char PROGMEM str_scheduler_arg1[] = "show";
const parse_token_string_t PROGMEM cmd_scheduler_arg1 = TOKEN_STRING_INITIALIZER(struct cmd_scheduler_result, arg1, str_scheduler_arg1);

const char PROGMEM help_scheduler[] = "Show scheduler events";
const parse_inst_t PROGMEM cmd_scheduler = {
	.f = cmd_scheduler_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_scheduler,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_scheduler_arg0,
		(PGM_P)&cmd_scheduler_arg1,
		NULL,
	},
};

/**********************************************************/
/* Log */

/* this structure is filled when cmd_log is parsed successfully */
struct cmd_log_result {
	fixed_string_t arg0;
	fixed_string_t arg1;
	uint8_t arg2;
	fixed_string_t arg3;
};

/* keep it sync with string choice */
static const char PROGMEM uart_log[] = "uart";
static const char PROGMEM i2c_log[] = "i2c";
static const char PROGMEM default_log[] = "default";

struct log_name_and_num {
	const char *name;
	uint8_t num;
};

static const struct log_name_and_num log_name_and_num[] = {
	{ uart_log, E_UART },
	{ i2c_log, E_I2C },
	{ default_log, E_USER_DEFAULT },
};

static uint8_t
log_name2num(const char * s)
{
	uint8_t i;

	for (i=0; i<sizeof(log_name_and_num)/sizeof(struct log_name_and_num); i++) {
		if (!strcmp_P(s, log_name_and_num[i].name)) {
			return log_name_and_num[i].num;
		}
	}
	return 0;
}

const char *
log_num2name(uint8_t num)
{
	uint8_t i;

	for (i=0; i<sizeof(log_name_and_num)/sizeof(struct log_name_and_num); i++) {
		if (num ==  log_name_and_num[i].num) {
			return log_name_and_num[i].name;
		}
	}
	return NULL;
}

/* function called when cmd_log is parsed successfully */
static void cmd_log_do_show(void)
{
	uint8_t i, empty=1;
	const char *name;

	printf_P(PSTR("log level is %d\r\n"), xbeeboard.log_level);
	for (i=0; i<NB_LOGS; i++) {
		name = log_num2name(xbeeboard.logs[i]);
		if (name) {
#ifdef HOST_VERSION
			printf_P(PSTR("log type %s is on\r\n"), name);
#else
			printf_P(PSTR("log type %S is on\r\n"), name);
#endif
			empty = 0;
		}
	}
	if (empty)
		printf_P(PSTR("no log configured\r\n"));
}

/* function called when cmd_log is parsed successfully */
static void cmd_log_parsed(void * parsed_result, void *data)
{
	struct cmd_log_result *res = (struct cmd_log_result *) parsed_result;

	(void)data;
	if (!strcmp_P(res->arg1, PSTR("level"))) {
		xbeeboard.log_level = res->arg2;
	}

	/* else it is a show */
	cmd_log_do_show();
}

const char PROGMEM str_log_arg0[] = "log";
const parse_token_string_t PROGMEM cmd_log_arg0 = TOKEN_STRING_INITIALIZER(struct cmd_log_result, arg0, str_log_arg0);
const char PROGMEM str_log_arg1[] = "level";
const parse_token_string_t PROGMEM cmd_log_arg1 = TOKEN_STRING_INITIALIZER(struct cmd_log_result, arg1, str_log_arg1);
const parse_token_num_t PROGMEM cmd_log_arg2 = TOKEN_NUM_INITIALIZER(struct cmd_log_result, arg2, INT8);

const char PROGMEM help_log[] = "Set log options: level (0 -> 5)";
const parse_inst_t PROGMEM cmd_log = {
	.f = cmd_log_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_log,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_log_arg0,
		(PGM_P)&cmd_log_arg1,
		(PGM_P)&cmd_log_arg2,
		NULL,
	},
};

const char PROGMEM str_log_arg1_show[] = "show";
const parse_token_string_t PROGMEM cmd_log_arg1_show = TOKEN_STRING_INITIALIZER(struct cmd_log_result, arg1, str_log_arg1_show);

const char PROGMEM help_log_show[] = "Show configured logs";
const parse_inst_t PROGMEM cmd_log_show = {
	.f = cmd_log_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_log_show,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_log_arg0,
		(PGM_P)&cmd_log_arg1_show,
		NULL,
	},
};

/* this structure is filled when cmd_log is parsed successfully */
struct cmd_log_type_result {
	fixed_string_t arg0;
	fixed_string_t arg1;
	fixed_string_t arg2;
	fixed_string_t arg3;
};

/* function called when cmd_log is parsed successfully */
static void cmd_log_type_parsed(void * parsed_result, void *data)
{
	struct cmd_log_type_result *res = (struct cmd_log_type_result *) parsed_result;
	uint8_t lognum;
	uint8_t i;

	(void)data;

	lognum = log_name2num(res->arg2);
	if (lognum == 0) {
		printf_P(PSTR("Cannot find log num\r\n"));
		return;
	}

	if (!strcmp_P(res->arg3, PSTR("on"))) {
		for (i=0; i<NB_LOGS; i++) {
			if (xbeeboard.logs[i] == lognum) {
				printf_P(PSTR("Already on\r\n"));
				return;
			}
		}
		for (i=0; i<NB_LOGS; i++) {
			if (xbeeboard.logs[i] == 0) {
				xbeeboard.logs[i] = lognum;
				break;
			}
		}
		if (i==NB_LOGS) {
			printf_P(PSTR("no more room\r\n"));
		}
	}
	else if (!strcmp_P(res->arg3, PSTR("off"))) {
		for (i=0; i<NB_LOGS; i++) {
			if (xbeeboard.logs[i] == lognum) {
				xbeeboard.logs[i] = 0;
				break;
			}
		}
		if (i==NB_LOGS) {
			printf_P(PSTR("already off\r\n"));
		}
	}
	cmd_log_do_show();
}

const char PROGMEM str_log_arg1_type[] = "type";
const parse_token_string_t PROGMEM cmd_log_arg1_type = TOKEN_STRING_INITIALIZER(struct cmd_log_type_result, arg1, str_log_arg1_type);
/* keep it sync with log_name_and_num above */
const char PROGMEM str_log_arg2_type[] = "uart#rs#servo#traj#i2c#oa#strat#i2cproto#ext#sensor#bd#cs";
const parse_token_string_t PROGMEM cmd_log_arg2_type = TOKEN_STRING_INITIALIZER(struct cmd_log_type_result, arg2, str_log_arg2_type);
const char PROGMEM str_log_arg3[] = "on#off";
const parse_token_string_t PROGMEM cmd_log_arg3 = TOKEN_STRING_INITIALIZER(struct cmd_log_type_result, arg3, str_log_arg3);

const char PROGMEM help_log_type[] = "Set log type";
const parse_inst_t PROGMEM cmd_log_type = {
	.f = cmd_log_type_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_log_type,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_log_arg0,
		(PGM_P)&cmd_log_arg1_type,
		(PGM_P)&cmd_log_arg2_type,
		(PGM_P)&cmd_log_arg3,
		NULL,
	},
};


/**********************************************************/
/* Stack_Space */

/* this structure is filled when cmd_stack_space is parsed successfully */
struct cmd_stack_space_result {
	fixed_string_t arg0;
};

/* function called when cmd_stack_space is parsed successfully */
static void cmd_stack_space_parsed(void *parsed_result, void *data)
{
	(void)parsed_result;
	(void)data;
#ifdef HOST_VERSION
	printf("not implemented\n");
#else
	printf("res stack: %d\r\n", min_stack_space_available());
#endif
}

const char PROGMEM str_stack_space_arg0[] = "stack_space";
const parse_token_string_t PROGMEM cmd_stack_space_arg0 = TOKEN_STRING_INITIALIZER(struct cmd_stack_space_result, arg0, str_stack_space_arg0);

const char PROGMEM help_stack_space[] = "Display remaining stack space";
const parse_inst_t PROGMEM cmd_stack_space = {
	.f = cmd_stack_space_parsed,  /* function to call */
	.data = NULL,      /* 2nd arg of func */
	.help_str = help_stack_space,
	.tokens = {        /* token list, NULL terminated */
		(PGM_P)&cmd_stack_space_arg0,
		NULL,
	},
};
